﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

using System.Windows.Forms;
using System.Drawing;

namespace GradientionSample
{
    class GradientionButton2 : System.Windows.Forms.Button 
    {
        // ボタンを描画します
        private void DrawButton(Graphics g, bool pressed, bool focused)
        {
            // ボタンの背景を描画
            Pen backLinePen;
            if (!focused)
            {
                backLinePen = new Pen(Color.Black);
            }
            else
            {
                // フォーカスが当たっている場合は、枠線を太く描画する
                backLinePen = new Pen(Color.Black, 2.5f);
            }

            // DrawRectangleメソッドで線を描画するが
            // 線の太さを考慮して幅と高さから引く
            Rectangle lineRect = new Rectangle();
            lineRect.X = ClientRectangle.X + (int)backLinePen.Width;
            lineRect.Y = ClientRectangle.Y + (int)backLinePen.Width;
            lineRect.Width = ClientRectangle.Width - (int)backLinePen.Width * 2;
            lineRect.Height = ClientRectangle.Height - (int)backLinePen.Width * 2;

            g.FillRectangle(new SolidBrush(Color.DimGray), ClientRectangle);
            g.DrawRectangle(backLinePen, lineRect);

            // ボタンの表示領域
            Rectangle btnRect = ClientRectangle;
            btnRect.X += 2;
            btnRect.Y += 2;
            btnRect.Width -= 4;
            btnRect.Height -= 4;

            Color stColor, edColor;
            if (!pressed)
            {
                stColor = Color.Red;
                edColor = Color.Black;
            }
            else
            {
                // タップされている場合は、色を反転させて描画する
                stColor = Color.Black;
                edColor = Color.Red;
            }

            // 角丸グラデーションボタンの描画
            GraphicsExtension.DrawGradientRoundRectButton(
                g, btnRect, stColor, edColor, Color.Black, FillDirection.TopToBottom);

            // ボタンに表示するテキストの表示(テキストは中央に表示)
            StringFormat sf = new StringFormat();
            sf.LineAlignment = StringAlignment.Center;
            sf.Alignment = StringAlignment.Center;
            g.DrawString(Text, Font, new SolidBrush(Color.White), btnRect, sf);
        }

        public GradientionButton2()
        {
            // WM_PAINTをフックして、WM_Paint_Handlerメソッドを呼び出す
            WndProcHooker.HookWndProc(this,
                new WndProcHooker.WndProcCallback(WM_Paint_Handler),
                Win32.WM_PAINT);

            // タップ(マウス操作の)イベントをフックして、
            // それぞれのメソッドを呼び出す
            WndProcHooker.HookWndProc(this,
                new WndProcHooker.WndProcCallback(WM_LButtonDown_Handler),
                Win32.WM_LBUTTONDOWN);
            WndProcHooker.HookWndProc(this,
                new WndProcHooker.WndProcCallback(WM_LButtonUp_Handler),
                Win32.WM_LBUTTONUP);

        }

        // WndProcがWM_PAINTを受け取った際のコールバック関数
        int WM_Paint_Handler(IntPtr hwnd, uint msg, 
            uint wParam, int lParam, ref bool handled)
        {
            Win32.PAINTSTRUCT ps = new Win32.PAINTSTRUCT();

            // 前回作成した描画メソッドの呼び出し
            Graphics gr = Graphics.FromHdc(Win32.BeginPaint(hwnd, ref ps));
            DrawButton(gr, this.Capture, this.Focused);
            gr.Dispose();
            Win32.EndPaint(hwnd, ref ps);

            handled = true;
            return 0;
        }

        int WM_LButtonDown_Handler(IntPtr hwnd, uint msg, 
            uint wParam, int lParam, ref bool handled)
        {
            this.Capture = true;
            this.Focus();

            IntPtr hdc = Win32.GetDC(hwnd);
            Graphics gr = Graphics.FromHdc(hdc);
            // draw the button
            DrawButton(gr, this.Capture, this.Focused);
            gr.Dispose();
            Win32.ReleaseDC(hwnd, hdc);

            handled = true;
            return 0;
        }

        int WM_LButtonUp_Handler(IntPtr hwnd, uint msg, uint wParam, int lParam, ref bool handled)
        {
            this.Capture = false;

            IntPtr hdc = Win32.GetDC(hwnd);
            Graphics gr = Graphics.FromHdc(hdc);
            // draw the button
            DrawButton(gr, this.Capture, this.Focused);
            gr.Dispose();
            Win32.ReleaseDC(hwnd, hdc);

            handled = true;
            return 0;
        }
    }
}
